require_relative 'cloud_runtime'
require 'uu/os/cmd/command_client'
require 'uu/os/security/session'

module UU
  module CloudRuntime
    # NodeImage class.
    class NodeImage < ::UU::CloudRuntime::RuntimeBaseClass

      # Current status of a NodeImage. This information refers to the time-point
      # when the information gets obtained and is not stored in any persistent
      # storage.
      #
      # @see https://stackoverflow.com/a/1162838
      class Status

        # The NodeImage is currently being used.
        USED = 'USED'
        # The Node Image is not currently being used.
        UNUSED = 'UNUSED'

      end

      # Remote service path.
      # @private
      PATH = '/uu-cloud-runtime/NodeImage'
      _init_command_client(PATH)

      # Creates a new instance of +NodeImage+.
      #
      # @param session [UU::OS::Security::Session] Instance of session representing authenticated user
      #   (can be obtained with {UU::OS::Security::Session#current_session} method).
      def initialize(session = nil)
        @client = UU::OS::CMD::CommandClient.new(PATH, session)
      end

      # Returns a paged list of NodeImages filterable by status.
      #
      # @param cloud_uri [UU::OS::UESURI, String] Cloud uri.
      # @param parameters [Hash] Filtering criteria.
      # @option parameters [Fixnum] pageIndex Index of paged list.
      # @option parameters [Fixnum] pageSize Maximum records per paged list.
      # @option parameters [UU::CloudRuntime::NodeImage::Status, String] status Filter records by current status.
      #   * USED - NodeImage is being used by running Node of application, Gateway or RuntimeStack.
      #   * UNUSED - NodeImage is not being used by any application, Gateway or RuntimeStack.
      # @return [Hash] Structured hash.
      # @see .get_node_image_list
      def get_node_image_list(cloud_uri, parameters = {})
        @client.invoke(:getNodeImageList, cloud_uri, parameters: parameters)
      end

      # Returns a paged list of NodeImages filterable by status.
      # @example
      #   require 'uu_c3'
      #   UU::OS::Security::Session.login('...')
      #   CLOUD_URI = UU::OS::UESURI.new("ues:OT-BT:PLUS4U_CLOUD")
      #
      #   list = UU::CloudRuntime::NodeImage.get_node_image_list(
      #     CLOUD_URI,
      #     :pageIndex => 0,
      #     :pageSize => 500,
      #     :status => UU::CloudRuntime::NodeImage::Status::UNUSED
      #   )
      #
      #   puts list[:totalSize]
      #   puts list[:criteria]
      #   list[:pageEntries].each do |node_image|
      #     puts node_image #something...
      #   end
      #
      #
      # @param cloud_uri [UU::OS::UESURI, String] Cloud uri.
      # @param parameters [Hash] Filtering criteria.
      # @option parameters [Fixnum] pageIndex Index of paged list.
      # @option parameters [Fixnum] pageSize Maximum records per paged list.
      # @option parameters [UU::CloudRuntime::NodeImage::Status, String] status Filter records by current status.
      #   * USED - NodeImage is being used by running Node of application, Gateway or RuntimeStack.
      #   * UNUSED - NodeImage is not being used by any application, Gateway or RuntimeStack.
      # @return [Hash] Structured hash.
      # @see .get_node_image_list
      def self.get_node_image_list(cloud_uri, parameters = {})
        self.new(UU::OS::Security::Session.current_session).get_node_image_list(cloud_uri, parameters)
      end

    end
  end
end
